/*
 * Copyright (c) 2013 Nathaniel Jones
 * Massachusetts Institute of Technology
 */

#pragma once

#include <optixu_vector_types.h>

/* Types of normal materials for the OptiX radiance_normal program. */
#define  SHADE_MAT_PLASTIC	0u		/* sun or other distant uniform source */
#define  SHADE_MAT_METAL	1u		/* cie sky model implemented in skybright.cal */
#define  SHADE_MAT_TRANS	2u		/* perez sky model implemented in perezlum.cal */

/* Types of sky that may be displayed with an OptiX miss program. */
#define  SKY_NONE	0u		/* sun or other distant uniform source */
#define  SKY_CIE	1u		/* cie sky model implemented in skybright.cal */
#define  SKY_PEREZ	2u		/* perez sky model implemented in perezlum.cal */

typedef struct struct_BasicLight
{
#if defined(__cplusplus)
  typedef optix::float3 float3;
#endif
  float3 pos;
  float3 color;
  float  solid_angle;
  unsigned int type;   /* Reference to the sky type defined above. */
  int    function;     /* Reference to the OptiX buffer index of the brightness function for this light. */
  int    casts_shadow;
} BasicLight;

struct TriangleLight
{
#if defined(__cplusplus)
  typedef optix::float3 float3;
#endif
  float3 v1, v2, v3;
  float3 normal;
  float3 emission;
};

/* Structure for the parameters necessary to run skybright.cal. */
typedef struct struct_SkyBright
{
#if defined(__cplusplus)
  typedef optix::float3 float3;
#endif
  int type;      /* 1 for CIE clear, 2 for CIE overcast, 3 for uniform, 4 for CIE intermediate */
  float zenith;  /* zenith brightness */
  float ground;  /* ground plane brightness */
  float factor;  /* normalization factor based on sun direction */
  float3 sun;    /* sun direction */
} SkyBright;

/* Structure for the parameters necessary to run perezlum.cal. */
typedef struct struct_PerezLum
{
#if defined(__cplusplus)
  typedef optix::float3 float3;
#endif
  float diffuse; /* diffuse normalization */
  float ground;  /* ground brightness */
  float coef[5]; /* coefficients for the Perez model */
  float3 sun;    /* sun direction */
} PerezLum;

/* Structure to hold an ambient record */
typedef struct struct_ambient_record
{
#if defined(__cplusplus)
	typedef optix::float3 float3;
#endif
	float3 pos;		/* position in space */
	float3 dir;		/* normal direction */
	float3 gpos;	/* gradient wrt. position */
	float3 gdir;	/* gradient wrt. direction */
	float3 val;		/* computed ambient value */
	float  weight;	/* weight of parent ray */
	float  rad;		/* validity radius */
	unsigned int lvl;	/* recursion level of parent ray */
} AmbientRecord;
