/*
 * Copyright (c) 2013 Nathaniel Jones
 * Massachusetts Institute of Technology
 */

#include <fvect.h>
#include <resolu.h>
#include <view.h>
#include <color.h>

#include <optix.h>
#include <sutil.h>


#define TIMEOUT_CALLBACK
#define DEBUG_OPTIX

char path_to_ptx[512];     /* The path to the PTX file. */

extern COLOR* color_output;
extern float* depth_output;

/* in optix_radiance.c */
extern void renderOptix(const VIEW view, const int width, const int height, const double alarm, COLOR* colors, float* depths);

/* in optix_ambient.c */
extern void createAmbientRecords( RTcontext context, const VIEW view, const int width, const int height );
extern void populateAmbientRecords( RTcontext context );
extern void createGeometryInstanceAmbient( RTcontext context, RTgeometryinstance* instance );
extern void createAmbientAcceleration( RTcontext context, RTgeometryinstance instance );
extern void iterativeAmbientCacheBuild( RTcontext context, RTgeometryinstance* instance, const VIEW view, const int width, const int height );

/* in optix_radiance.c */
void applyContextVariable1ui( RTcontext context, char* name, unsigned int value );
void applyContextVariable1i( RTcontext context, char* name, int value );
void applyContextVariable1f( RTcontext context, char* name, float value );
void applyContextVariable3f( RTcontext context, char* name, float x, float y, float z );
void applyProgramVariable1ui( RTcontext context, RTprogram program, char* name, unsigned int value );
void applyProgramVariable1f( RTcontext context, RTprogram program, char* name, float value );
void applyProgramVariable2f( RTcontext context, RTprogram program, char* name, float x, float y );
void applyProgramVariable3f( RTcontext context, RTprogram program, char* name, float x, float y, float z );
void applyMaterialVariable1ui( RTcontext context, RTmaterial material, char* name, unsigned int value );
void applyMaterialVariable1f( RTcontext context, RTmaterial material, char* name, float value );
void applyMaterialVariable3f( RTcontext context, RTmaterial material, char* name, float x, float y, float z );
void createBuffer1D( RTcontext context, RTbuffertype type, RTformat format, int element_count, RTbuffer* buffer );
void createCustomBuffer1D( RTcontext context, RTbuffertype type, int element_size, int element_count, RTbuffer* buffer );
void createBuffer2D( RTcontext context, RTbuffertype type, RTformat format, int x_count, int y_count, RTbuffer* buffer );
void createCustomBuffer2D( RTcontext context, RTbuffertype type, int element_size, int x_count, int y_count, RTbuffer* buffer );
void createBuffer3D( RTcontext context, RTbuffertype type, RTformat format, int x_count, int y_count, int z_count, RTbuffer* buffer );
void createCustomBuffer3D( RTcontext context, RTbuffertype type, int element_size, int x_count, int y_count, int z_count, RTbuffer* buffer );
void applyContextObject( RTcontext context, char* name, RTobject object );
void applyGeometryObject( RTcontext context, RTgeometry geometry, char* name, RTobject object );
#ifdef TIMEOUT_CALLBACK
static int timeoutCallback(void);
#endif
